# -*- coding: utf-8 -*-
"""Config file for Sphinx-docs."""
from __future__ import unicode_literals

import os


extensions = [
    "myst_parser",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.coverage",
    "sphinx.ext.doctest",
    "sphinx.ext.extlinks",
    "sphinx.ext.ifconfig",
    "sphinx.ext.napoleon",
    "sphinx.ext.todo",
    "sphinx.ext.viewcode",
    "sphinx.ext.autosectionlabel",
]

todo_include_todos = True

exclude_patterns = []

if os.getenv("SPELLCHECK"):
    extensions += ("sphinxcontrib.spelling",)
    spelling_show_suggestions = True
    spelling_lang = "en_US"
    # https://sphinxcontrib-spelling.readthedocs.io/en/latest/customize.html
    spelling_word_list_filename = ["spelling_wordlist.txt"]

source_suffix = ".rst"
master_doc = "index"
project = ""
year = "2023"
author = "Daan van Vugt"
copyright = "{0}, {1}".format(year, author)
version = release = "0.1.1"

pygments_style = "trac"
templates_path = ["."]
extlinks = {
    "issue": ("https://github.com/DaanVanVugt/ymmsl-dot/issues/%s", "#"),  # noqa: E501
    "pr": ("https://github.com/DaanVanVugt/ymmsl-dot/pull/%s", "PR #"),  # noqa: E501
}

# codecov io closes connection if host is accessed too repetitively.
# codecov links are ignored here for the same reason there's a sleep
# in the .travis.yml file
# see https://github.com/codecov/codecov-python/issues/158
linkcheck_ignore = [
    r"https://codecov.io/gh/DaanVanVugt/ymmsl-dot/*",
]

html_theme_options = {}

html_use_smartypants = True
html_last_updated_fmt = "%b %d, %Y"
html_split_index = False
html_sidebars = {
    "**": ["searchbox.html", "globaltoc.html", "sourcelink.html"],
}
html_short_title = "%s-%s" % (project, version)

napoleon_use_ivar = True
napoleon_use_rtype = False
napoleon_use_param = False



import logging
from typing import List, Union

import pydot
from ymmsl.component import Component, Operator
from ymmsl.configuration import PartialConfiguration
from ymmsl.identity import Reference


COLORS = {
    Operator.F_INIT: "#2998ba",
    Operator.O_I: "#eddea1",
    Operator.S: "#f1c40f",
    Operator.O_F: "#e67e22",
}


def port_operator(port: Reference, component: Union[Component, None]):
    """Look up the operator corresponding to a specific port"""
    return (
        component.ports.operator(port)
        if component and component.ports
        else Operator.NONE
    )


def port_shape(operator: str, simple: bool = True):
    """Given a port reference, find the component referred to,
    look up the port type matching this name
    and look up the shape corresponding to the port type."""

    if simple:
        if operator == Operator.F_INIT or operator == Operator.S:
            return "normal"
        return "none"
    else:
        # I think it is quite easy to misinterpret the direction of a MMSL diagram
        # given that the 'weight' of the edge is towards the filled shape.
        # I would consider making the sending_port smaller, but cannot find the setting
        # in graphviz
        # https://www.graphviz.org/docs/attr-types/arrowType/
        OPERATOR_SHAPES = {
            Operator.NONE: "none",
            Operator.F_INIT: "odiamond",
            Operator.O_I: "dot",
            Operator.S: "odot",
            Operator.O_F: "diamond",
        }
        return OPERATOR_SHAPES[operator]


def find_component(name: Reference, components: List[Component]):
    """Find a component by reference"""
    return next(
        (component for component in components if component.name == str(name)), None
    )


def set_style(graph, draw_ports: bool = False):
    """set default properties to make for a more readable DOT file"""
    graph.add_node(
        pydot.Node(
            "node",
            shape="plain" if draw_ports else "box",
            style="rounded",
            fixedsize="false",
            width=2,
            penwidth=2,
            height=1,
            labelloc="c",
        )
    )

    # set default edge properties
    graph.add_node(
        pydot.Node(
            "edge",
            dir="both",
            labelfontsize=10,
            fontsize=10,
            penwidth=2,
            len=2,
        )
    )


def trim_sending_port(identifier: str):
    """Strip _out suffix from identifier"""
    return identifier[:-4] if identifier.endswith("_out") else identifier


def trim_receiving_port(identifier: str):
    """Strip _in, _init suffix from identifier"""
    if identifier.endswith("_in"):
        return identifier[:-3]
    if identifier.endswith("_init"):
        return identifier[:-5]
    return identifier


def headport(identifier: Reference, component: Union[Component, None]):
    """Given a reference, return the portPos.
    https://www.graphviz.org/docs/attr-types/portPos/
    """
    return str(identifier)


def tailport(identifier: Reference, component: Union[Component, None]):
    """Given a reference, return the portPos.
    https://www.graphviz.org/docs/attr-types/portPos/
    """
    return str(identifier)


def port_shortname(identifier: Reference):
    """Strip suffixes and summarize names to only a few characters"""
    identifier = trim_sending_port(trim_receiving_port(str(identifier)))
    return "".join([s[0] for s in identifier.split("_")]).upper()


def legend_html_label():
    return f"""<<TABLE CELLSPACING="0" CELLBORDER="0" >
  <TR>
    <TD BGCOLOR='{COLORS[Operator.F_INIT]}'>F_INIT</TD>
    <TD BGCOLOR='{COLORS[Operator.O_I]}'>O_I</TD>
    <TD><B>legend</B></TD>
    <TD BGCOLOR='{COLORS[Operator.S]}'>S</TD>
    <TD BGCOLOR='{COLORS[Operator.O_F]}'>O_F</TD>
  </TR>
</TABLE>>"""
    pass


def component_html_label(component: Component):
    """Construct a HTML-like label (https://graphviz.org/doc/info/shapes.html#html)"""

    if component.ports is None:
        logging.warning("no ports defined on component, skipping.")
        return component.name

    label = "<<TABLE CELLSPACING='0' CELLBORDER='0' >\n  <TR>\n"

    for port in component.ports.f_init:
        label += f"    <TD PORT='{port}' BGCOLOR='{COLORS[Operator.F_INIT]}'\
            >{port_shortname(port)}</TD>\n"
    if len(component.ports.f_init) == 0:
        label += f"    <TD BGCOLOR='{COLORS[Operator.F_INIT]}'></TD>\n"
    for port in component.ports.o_i:
        label += f"    <TD PORT='{port}' BGCOLOR='{COLORS[Operator.O_I]}'\
            >{port_shortname(port)}</TD>\n"
    if len(component.ports.o_i) == 0:
        label += f"    <TD BGCOLOR='{COLORS[Operator.O_I]}'></TD>\n"

    label += f"    <TD><B>{component.name}</B></TD>\n"

    for port in component.ports.s:
        label += f"    <TD PORT='{port}' BGCOLOR='{COLORS[Operator.S]}'\
            >{port_shortname(port)}</TD>\n"
    if len(component.ports.s) == 0:
        label += f"    <TD BGCOLOR='{COLORS[Operator.S]}'></TD>\n"
    for port in component.ports.o_f:
        label += f"    <TD PORT='{port}' BGCOLOR='{COLORS[Operator.O_F]}'\
            >{port_shortname(port)}</TD>\n"
    if len(component.ports.o_f) == 0:
        label += f"    <TD BGCOLOR='{COLORS[Operator.O_F]}'></TD>\n"

    return label.replace("'", '"') + "</TR></TABLE>>"


def plot_model_graph(
    config: PartialConfiguration,
    simplify_edge_labels: bool = True,
    draw_ports: bool = False,
    simple_edges: bool = True,
    show_legend: bool = True,
) -> None:
    """Convert a PartialConfiguration into DOT format."""
    graph = pydot.Dot(
        config.model.name,
        graph_type="digraph",
        layout="dot",
        pad=1,
        # splines="ortho",
        nodesep=0.6,
        ranksep=0.75,
        fontname="Sans-Serif",
    )
    # be very careful with ortho splines, I have seen it put edges
    # upside down and eating labels
    set_style(graph, draw_ports=draw_ports)

    if draw_ports:
        label_method = component_html_label
    else:
        def label_method(x):
            return x.name

    # Start with a legend node
    if draw_ports and show_legend:
        graph.add_node(pydot.Node("legend", label=legend_html_label()))

    for component in config.model.components:
        graph.add_node(pydot.Node(str(component.name), label=label_method(component)))

    # assume that conduits are ordered by port
    last_port = None
    for conduit in config.model.conduits:
        # emit an edge[sametail=] config for this port name
        # if it is changed from the previous one
        # note that this does not work for self-loops:
        # https://gitlab.com/graphviz/graphviz/-/blob/main/lib/dotgen/sameport.c#L53
        if str(conduit.sending_port()) != last_port:
            graph.add_node(pydot.Node("edge", sametail=str(conduit.sending_port())))
            last_port = str(conduit.sending_port())

        # can we do this more elegantly?
        sender = find_component(conduit.sending_component(), config.model.components)
        receiver = find_component(
            conduit.receiving_component(), config.model.components
        )

        # Due to yMMSL conventions we cannot have edges with the same head (since
        # an input port can only have one conduit connected to it)
        port_config = {
            "tailport": tailport(conduit.sending_port(), sender),
            "headport": headport(conduit.receiving_port(), receiver),
        }

        edge = pydot.Edge(
            str(conduit.sending_component()),
            str(conduit.receiving_component()),
            arrowtail=port_shape(
                port_operator(
                    conduit.sending_port(),
                    sender,
                ),
                simple_edges,
            ),
            arrowhead=port_shape(
                port_operator(
                    conduit.receiving_port(),
                    receiver,
                ),
                simple_edges,
            ),
            **port_config,
        )

        # if port names match exactly (optionally when removing an _in or _out suffix)
        # we show the name on the conduit instead of on the port
        if simplify_edge_labels and trim_sending_port(
            str(conduit.sending_port())
        ) == trim_receiving_port(str(conduit.receiving_port())):
            edge.set_label(trim_sending_port(str(conduit.sending_port())))
        else:
            edge.set_taillabel(str(conduit.sending_port()))
            edge.set_headlabel(str(conduit.receiving_port()))

        # we could consider setting a minlen based on the text length and font size
        graph.add_edge(edge)

    return graph



"""Command-line tools for ymmsl."""
import subprocess
from pathlib import Path
from typing import Sequence, Union

import click
import pydot
import ymmsl
from ymmsl import PartialConfiguration

from .model_to_dot import plot_model_graph


@click.group()
def main() -> None:
    """This command provides various functions for analysing yMMSL coupled simulations.

    Use ymmsl <command> --help for help with individual commands.
    """
    pass


@main.command(short_help="Print a graphical representation of this workflow")
@click.argument(
    "ymmsl_files",
    nargs=-1,
    required=True,
    type=click.Path(
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        allow_dash=True,
        resolve_path=True,
    ),
)
@click.option(
    "-o",
    "--out",
    nargs=1,
    required=False,
    type=click.Path(
        exists=False,
        file_okay=True,
        dir_okay=False,
        readable=False,
        allow_dash=True,
        resolve_path=True,
    ),
    help="Output file (default ./output.format)",
)
@click.option(
    "-f",
    "--fmt",
    type=click.Choice(pydot.Dot().formats, case_sensitive=False),
    required=False,
    help="Set output format (default svg).",
)
@click.option(
    "-w",
    "--viewer",
    nargs=1,
    required=False,
    type=str,
    help="Open with specified viewer (try xdg-open)",
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    help="Show more info (prints the generated dot syntax)",
)
@click.option("-p", "--ports", is_flag=True, help="Explicitly draw component ports.")
@click.option("-l", "--legend", is_flag=True, help="Show a legend (only with --ports).")
@click.option(
    "-s",
    "--simple-edges",
    is_flag=True,
    help="Only indicate conduit direction, not port types.",
)
@click.option(
    "--portlabels",
    is_flag=True,
    help="Never simplify matching port labels along an edge.",
)
def graph(
    ymmsl_files: Sequence[str],
    out: Union[Path, None],
    fmt: str,
    viewer: str,
    verbose: bool,
    ports: bool,
    legend: bool,
    simple_edges: bool,
    portlabels: bool,
) -> None:
    """Plot a graphical representation of the passed yMMSL files.

    To help develop or understand about a coupled simulation it may
    be useful to view a graphical representation.

    If multiple yMMSL files are given, then they will be combined left
    to right, i.e. if there are conflicting declarations, the one from
    the file last given is used.

    Result:

      A graph is displayed or saved to disk, containing all the defined
    components and the connections between them.

    Examples:

      muscle3 graph simulation.ymmsl

    """
    partial_config = _load_ymmsl_files(ymmsl_files)

    graph = plot_model_graph(
        partial_config,
        simplify_edge_labels=not portlabels,
        draw_ports=ports,
        simple_edges=simple_edges,
        show_legend=legend,
    )

    if fmt is None:
        fmt = "svg"
    if out is None:
        out = "output." + fmt

    if out == "-" and fmt == "dot":
        print(graph)  # noqa:T201
    elif out == "-":
        print(graph.create(format=fmt))  # noqa:T201
    else:
        if verbose:
            print(graph)  # noqa:T201

        graph.write(out, format=fmt)

    if viewer is not None and out != "-":
        subprocess.run([viewer, out])


def _load_ymmsl_files(ymmsl_files: Sequence[str]) -> PartialConfiguration:
    """Load and merge yMMSL files."""
    configuration = PartialConfiguration()
    for path in ymmsl_files:
        with open(path, "r") as f:
            configuration.update(ymmsl.load(f))
    return configuration


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""Setup dot py."""
from __future__ import absolute_import, print_function

# import re
from glob import glob
from os.path import basename, dirname, join, splitext

from setuptools import find_packages, setup


def read(*names, **kwargs):
    """Read description files."""
    path = join(dirname(__file__), *names)
    with open(path, encoding=kwargs.get("encoding", "utf8")) as fh:
        return fh.read()


# replace relative url with an URL to github
long_description = read("README.md").replace('./docs', 'https://github.com/DaanVanVugt/ymmsl-dot/raw/main/docs')

setup(
    name="ymmsl-dot",
    version="0.1.1",
    description="Visualise yMMSL models with graphviz.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="Apache 2.0",
    author="Daan van Vugt",
    author_email="dvanvugt@ignitioncomputing.com",
    url="https://github.com/DaanVanVugt/ymmsl-dot",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[splitext(basename(i))[0] for i in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list:
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Operating System :: Microsoft",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
    project_urls={
        "webpage": "https://github.com/DaanVanVugt/ymmsl-dot",
        "Documentation": "https://ymmsl-dot.readthedocs.io/en/latest/",
        "Issue Tracker": "https://github.com/DaanVanVugt/ymmsl-dot/issues",
        "Discussion Forum": "https://github.com/DaanVanVugt/ymmsl-dot/discussions",
    },
    keywords=[
        "ymmsl",
        "simulation",
        "graphviz",
        "dot",
        "muscle3",
        "yMMSL",
        "multiscale",
    ],
    python_requires=">=3.7, <4",
    install_requires=[
        # https://stackoverflow.com/questions/14399534
        "pydot>=1",
        "ymmsl",
        "click",
    ],
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    setup_requires=[
        #   'pytest-runner',
        #   'setuptools_scm>=3.3.1',
    ],
    entry_points={
        "console_scripts": [
            "ymmsl = ymmsl_dot.main:main",
        ]
        #
    },
)
